#!c:/perl/bin/perl.exe
use warnings;
use strict;
use diagnostics;
$| = 1;


my %tissues;

read_tissue_data();

while (1) {
	print "Enter tissue name:";
	my $tissue = <STDIN>;
	chomp $tissue;
	
	my @matches = get_tissues ($tissue);
	
	if (@matches == 0) {
		print "No tissues matched\n";
	}
	elsif (@matches == 1) {
		print "Found tissue '",$matches[0]->{term},"\n";
		foreach my $key (sort {$a cmp $b} keys (%{$matches[0]})) {
			print "\t$key\t",$matches[0]->{$key},"\n";	
		}			
	}
	else {
		print "Options are:\n";
		foreach my $match (@matches) {
			print "'",$match->{term},"' ";	
		}	
		print "\n";
	}
}


sub read_tissue_data {

	open (my $in, "Resources/tissues.txt") or die "Can't read tissue file: $!";
	my %tissue_details;
	my $tissue_name;
	while (<$in>) {
		chomp;
		next if (/^!/);
		
		# If we hit a blank line we're reached the end
		# of an entry and we need to enter it into the
		# larger data structure and reset the variables
		# we're using
		
		unless ($_) {
			
			# Here we need to create a new anonymous hash from the
			# existing hash.  We can't just make a reference to 
			# %tissue_details as all the entries would point to the
			# same hash.
			
			$tissues{$tissue_name} = {%tissue_details};
			$tissue_name = '';
			%tissue_details = ();
			next;
		}	
		
		# We split each data line at the first colon (plus any spaces)
		# the extra arguments to split here say that we only ever want
		# two values, so the second value should contain everything to
		# the end of the string, even if it contains another colon.
		my ($name,$value) = split(/:\s*/,$_,2);
				
		# We keep the name if we're looking at the term
		if ($name eq 'term') {
			# As we're going to need to query in a case insensitive
			# mannar we'll put the term in in lower case 
			$tissue_name = lc($value);
		}

		# We store all the details in the hash
		$tissue_details{$name} = $value;	
		
	}	
	
	# No need to return anything as we're populating a
	# globally accessible variable
}


sub get_tissues {
	
	my ($string) = @_;
	
	# See if we can find an exact match
	if (exists ($tissues{lc($string)})) {
		return $tissues{lc($string)};
	}
	
	# Otherwise return a list of possibilities
	my @matches;
	foreach my $term (sort {$a cmp $b} keys %tissues) {
		
		# We match based on our query equalling the front
		# of the term we're looking at.  We can't match if
		# our query is longer than the term.
				
		if ((length $term >= length $string) and (substr($term,0,length $string) eq lc($string))) {
			push @matches, $tissues{$term};
		}
	}
	
	return @matches;
			
}
